from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.pool_service_coverage_check_response_200 import (
    PoolService_CoverageCheckResponse_200,
)
from ...models.pool_service_coverage_check_response_401 import (
    PoolService_CoverageCheckResponse_401,
)
from ...models.pool_service_coverage_check_response_404 import (
    PoolService_CoverageCheckResponse_404,
)
from ...models.pool_service_coverage_check_response_500 import (
    PoolService_CoverageCheckResponse_500,
)
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: Client,
    cusips: List[str],
) -> Dict[str, Any]:
    url = "{}/api/v1/pools/coverage_check".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_cusips = cusips

    params: Dict[str, Any] = {
        "CUSIPS": json_cusips,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolService_CoverageCheckResponse_200.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = PoolService_CoverageCheckResponse_401.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolService_CoverageCheckResponse_404.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolService_CoverageCheckResponse_500.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cusips: List[str],
) -> Response[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cusips: List[str],
) -> Optional[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    """


    Args:
        cusips: CUSIPS to check coverage for.

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cusips=cusips,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cusips: List[str],
) -> Response[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cusips: List[str],
) -> Optional[
    Union[
        PoolService_CoverageCheckResponse_200,
        PoolService_CoverageCheckResponse_401,
        PoolService_CoverageCheckResponse_404,
        PoolService_CoverageCheckResponse_500,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            cusips=cusips,
        )
    ).parsed
