from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.pool_historical_prepayments import PoolHistoricalPrepayments
from ...models.pool_historical_prepayments_error import PoolHistoricalPrepaymentsError
from ...models.pool_historical_prepayments_not_found import (
    PoolHistoricalPrepaymentsNotFound,
)
from ...models.pool_historical_prepayments_unauthorized import (
    PoolHistoricalPrepaymentsUnauthorized,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    cusips: List[str],
    dateyear: Union[Unset, int] = UNSET,
    datemonth: Union[Unset, int] = UNSET,
    dateday: Union[Unset, int] = UNSET,
    num_months: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/pools/prepayment_history".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_cusips = cusips

    params: Dict[str, Any] = {
        "CUSIPS": json_cusips,
        "date.year": dateyear,
        "date.month": datemonth,
        "date.day": dateday,
        "numMonths": num_months,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    if response.status_code == 200:
        response_200 = PoolHistoricalPrepayments.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = PoolHistoricalPrepaymentsUnauthorized.from_dict(response.json())

        return response_401
    if response.status_code == 404:
        response_404 = PoolHistoricalPrepaymentsNotFound.from_dict(response.json())

        return response_404
    if response.status_code == 500:
        response_500 = PoolHistoricalPrepaymentsError.from_dict(response.json())

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    cusips: List[str],
    dateyear: Union[Unset, int] = UNSET,
    datemonth: Union[Unset, int] = UNSET,
    dateday: Union[Unset, int] = UNSET,
    num_months: Union[Unset, int] = UNSET,
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        dateyear=dateyear,
        datemonth=datemonth,
        dateday=dateday,
        num_months=num_months,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    cusips: List[str],
    dateyear: Union[Unset, int] = UNSET,
    datemonth: Union[Unset, int] = UNSET,
    dateday: Union[Unset, int] = UNSET,
    num_months: Union[Unset, int] = UNSET,
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    """


    Args:
        cusips: Cohort to fetch.
        dateyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
        datemonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        dateday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
        num_months: Number of months of historical data. Default to 5

    Returns:
        Either a valid response or an unauthorized, not found or other error.
    """

    return sync_detailed(
        client=client,
        cusips=cusips,
        dateyear=dateyear,
        datemonth=datemonth,
        dateday=dateday,
        num_months=num_months,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    cusips: List[str],
    dateyear: Union[Unset, int] = UNSET,
    datemonth: Union[Unset, int] = UNSET,
    dateday: Union[Unset, int] = UNSET,
    num_months: Union[Unset, int] = UNSET,
) -> Response[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    kwargs = _get_kwargs(
        client=client,
        cusips=cusips,
        dateyear=dateyear,
        datemonth=datemonth,
        dateday=dateday,
        num_months=num_months,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    cusips: List[str],
    dateyear: Union[Unset, int] = UNSET,
    datemonth: Union[Unset, int] = UNSET,
    dateday: Union[Unset, int] = UNSET,
    num_months: Union[Unset, int] = UNSET,
) -> Optional[
    Union[
        PoolHistoricalPrepayments,
        PoolHistoricalPrepaymentsUnauthorized,
        PoolHistoricalPrepaymentsNotFound,
        PoolHistoricalPrepaymentsError,
    ]
]:
    """  """

    return (
        await asyncio_detailed(
            client=client,
            cusips=cusips,
            dateyear=dateyear,
            datemonth=datemonth,
            dateday=dateday,
            num_months=num_months,
        )
    ).parsed
