from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CohortPoolComponent")


@attr.s(auto_attribs=True)
class CohortPoolComponent:
    """  """

    cusip: Union[Unset, str] = UNSET
    agency: Union[Unset, str] = UNSET
    current_upb: Union[Unset, float] = UNSET
    current_wac: Union[Unset, float] = UNSET
    issue_amount: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cusip = self.cusip
        agency = self.agency
        current_upb = self.current_upb
        current_wac = self.current_wac
        issue_amount = self.issue_amount

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cusip is not UNSET:
            field_dict["CUSIP"] = cusip
        if agency is not UNSET:
            field_dict["agency"] = agency
        if current_upb is not UNSET:
            field_dict["currentUpb"] = current_upb
        if current_wac is not UNSET:
            field_dict["currentWac"] = current_wac
        if issue_amount is not UNSET:
            field_dict["issueAmount"] = issue_amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cusip = d.pop("CUSIP", UNSET)

        agency = d.pop("agency", UNSET)

        current_upb = d.pop("currentUpb", UNSET)

        current_wac = d.pop("currentWac", UNSET)

        issue_amount = d.pop("issueAmount", UNSET)

        cohort_pool_component = cls(
            cusip=cusip,
            agency=agency,
            current_upb=current_upb,
            current_wac=current_wac,
            issue_amount=issue_amount,
        )

        cohort_pool_component.additional_properties = d
        return cohort_pool_component

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
