from typing import Dict, List, Optional, Union

import infima_client.api.pool_service.pool_service_coverage_check as pool_service_coverage_check
import infima_client.api.pool_service.pool_service_get_characteristics as pool_service_get_characteristics
import infima_client.api.pool_service.pool_service_get_distributions as pool_service_get_distributions
import infima_client.api.pool_service.pool_service_get_geolocations as pool_service_get_geolocations
import infima_client.api.pool_service.pool_service_get_historical_prepayments as pool_service_get_historical_prepayments
import infima_client.api.pool_service.pool_service_get_pools as pool_service_get_pools
import infima_client.api.pool_service.pool_service_get_quartiles as pool_service_get_quartiles
import infima_client.api.pool_service.pool_service_get_servicers as pool_service_get_servicers
import infima_client.api.pool_service.pool_service_get_summaries as pool_service_get_summaries
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset


class pool:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = poolDetailed(client)

    def coverage_check(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolService_CoverageCheckResponse_200,
            PoolService_CoverageCheckResponse_401,
            PoolService_CoverageCheckResponse_404,
            PoolService_CoverageCheckResponse_500,
        ]
    ]:

        """


        Args:
            cusips: CUSIPS to check coverage for.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_coverage_check.sync(client=self.client, cusips=cusips)

    def get_characteristics(
        self,
        cusips: Union[Unset, List[str]] = UNSET,
        dateyear: Union[Unset, int] = UNSET,
        datemonth: Union[Unset, int] = UNSET,
        dateday: Union[Unset, int] = UNSET,
    ) -> Optional[
        Union[
            PoolCharacteristics,
            PoolCharacteristicsUnauthorized,
            PoolCharacteristicsNotFound,
            PoolCharacteristicsError,
        ]
    ]:

        """


        Args:
            cusips: None
            dateyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            datemonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            dateday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_characteristics.sync(
            client=self.client,
            cusips=cusips,
            dateyear=dateyear,
            datemonth=datemonth,
            dateday=dateday,
        )

    def get_distributions(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolDistributions,
            PoolDistributionsUnauthorized,
            PoolDistributionsNotFound,
            PoolDistributionsError,
        ]
    ]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_distributions.sync(client=self.client, cusips=cusips)

    def get_geolocations(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolGeoLocations,
            PoolGeoLocationsUnauthorized,
            PoolGeoLocationsNotFound,
            PoolGeoLocationsError,
        ]
    ]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_geolocations.sync(client=self.client, cusips=cusips)

    def get_historical_prepayments(
        self,
        cusips: List[str],
        dateyear: Union[Unset, int] = UNSET,
        datemonth: Union[Unset, int] = UNSET,
        dateday: Union[Unset, int] = UNSET,
        num_months: Union[Unset, int] = UNSET,
    ) -> Optional[
        Union[
            PoolHistoricalPrepayments,
            PoolHistoricalPrepaymentsUnauthorized,
            PoolHistoricalPrepaymentsNotFound,
            PoolHistoricalPrepaymentsError,
        ]
    ]:

        """


        Args:
            cusips: Cohort to fetch.
            dateyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            datemonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            dateday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            num_months: Number of months of historical data. Default to 5

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_historical_prepayments.sync(
            client=self.client,
            cusips=cusips,
            dateyear=dateyear,
            datemonth=datemonth,
            dateday=dateday,
            num_months=num_months,
        )

    def get_pools(
        self, cusips: List[str]
    ) -> Optional[Union[Pool, PoolUnauthorized, PoolNotFound, PoolError]]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_pools.sync(client=self.client, cusips=cusips)

    def get_quartiles(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolQuartiles,
            PoolQuartilesUnauthorized,
            PoolQuartilesNotFound,
            PoolQuartilesError,
        ]
    ]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_quartiles.sync(client=self.client, cusips=cusips)

    def get_servicers(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolServicers,
            PoolServicersUnauthorized,
            PoolServicersNotFound,
            PoolServicersError,
        ]
    ]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_servicers.sync(client=self.client, cusips=cusips)

    def get_summaries(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            PoolSummaries,
            PoolSummariesUnauthorized,
            PoolSummariesNotFound,
            PoolSummariesError,
        ]
    ]:

        """


        Args:
            cusips: Pool CUSIPS to fetch.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return pool_service_get_summaries.sync(client=self.client, cusips=cusips)


class poolDetailed:
    def __init__(self, client: Client):
        self.client = client

    def coverage_check(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolService_CoverageCheckResponse_200,
            PoolService_CoverageCheckResponse_401,
            PoolService_CoverageCheckResponse_404,
            PoolService_CoverageCheckResponse_500,
        ]
    ]:

        return pool_service_coverage_check.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_characteristics(
        self,
        cusips: Union[Unset, List[str]] = UNSET,
        dateyear: Union[Unset, int] = UNSET,
        datemonth: Union[Unset, int] = UNSET,
        dateday: Union[Unset, int] = UNSET,
    ) -> Response[
        Union[
            PoolCharacteristics,
            PoolCharacteristicsUnauthorized,
            PoolCharacteristicsNotFound,
            PoolCharacteristicsError,
        ]
    ]:

        return pool_service_get_characteristics.sync_detailed(
            client=self.client,
            cusips=cusips,
            dateyear=dateyear,
            datemonth=datemonth,
            dateday=dateday,
        )

    def get_distributions(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolDistributions,
            PoolDistributionsUnauthorized,
            PoolDistributionsNotFound,
            PoolDistributionsError,
        ]
    ]:

        return pool_service_get_distributions.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_geolocations(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolGeoLocations,
            PoolGeoLocationsUnauthorized,
            PoolGeoLocationsNotFound,
            PoolGeoLocationsError,
        ]
    ]:

        return pool_service_get_geolocations.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_historical_prepayments(
        self,
        cusips: List[str],
        dateyear: Union[Unset, int] = UNSET,
        datemonth: Union[Unset, int] = UNSET,
        dateday: Union[Unset, int] = UNSET,
        num_months: Union[Unset, int] = UNSET,
    ) -> Response[
        Union[
            PoolHistoricalPrepayments,
            PoolHistoricalPrepaymentsUnauthorized,
            PoolHistoricalPrepaymentsNotFound,
            PoolHistoricalPrepaymentsError,
        ]
    ]:

        return pool_service_get_historical_prepayments.sync_detailed(
            client=self.client,
            cusips=cusips,
            dateyear=dateyear,
            datemonth=datemonth,
            dateday=dateday,
            num_months=num_months,
        )

    def get_pools(
        self, cusips: List[str]
    ) -> Response[Union[Pool, PoolUnauthorized, PoolNotFound, PoolError]]:

        return pool_service_get_pools.sync_detailed(client=self.client, cusips=cusips)

    def get_quartiles(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolQuartiles,
            PoolQuartilesUnauthorized,
            PoolQuartilesNotFound,
            PoolQuartilesError,
        ]
    ]:

        return pool_service_get_quartiles.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_servicers(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolServicers,
            PoolServicersUnauthorized,
            PoolServicersNotFound,
            PoolServicersError,
        ]
    ]:

        return pool_service_get_servicers.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_summaries(
        self, cusips: List[str]
    ) -> Response[
        Union[
            PoolSummaries,
            PoolSummariesUnauthorized,
            PoolSummariesNotFound,
            PoolSummariesError,
        ]
    ]:

        return pool_service_get_summaries.sync_detailed(
            client=self.client, cusips=cusips
        )
